/**
 *
 * Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.apache.geronimo.security.deploy;

import java.io.Serializable;


/**
 * @version $Revision: 355877 $ $Date: 2005-12-10 18:48:27 -0800 (Sat, 10 Dec 2005) $
 */
public class DistinguishedName implements Serializable {

    private final String name;
    private final boolean designatedRunAs;

    public DistinguishedName(String name, boolean designatedRunAs) {
        if (name == null) throw new IllegalArgumentException("name is null");

        this.name = name;
        this.designatedRunAs = designatedRunAs;
    }

    public String getName() {
        return name;
    }

    public boolean isDesignatedRunAs() {
        return designatedRunAs;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final DistinguishedName that = (DistinguishedName) o;

        if (designatedRunAs != that.designatedRunAs) return false;
        if (!name.equals(that.name)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = name.hashCode();
        result = 29 * result + (designatedRunAs ? 1 : 0);
        return result;
    }
}
